const path = require('path')
const fs = require('fs-extra')
const VendorExtracter = require('./extracter')
const chalk = require('chalk')

module.exports = extract

function extract (opts) {
  const entries = Array.isArray(opts.entries) ? opts.entries : []
  const fileModuleFomatter = opts.fileModuleFomatter || (i => i)
  const moduleFilter = opts.moduleFilter || (_ => true)
  const appendVersion = opts.appendVersion

  // webpack 相关的配置
  const vendorPrefix = opts.prefix || 'gallery/'
  const vendorFileNameTemplate = opts.fileNameTemplate || 'pkg.[name].js'
  const loaders = opts.loaders || []
  const prodMode = opts.prodMode || false

  const tempDirectory = path.join(process.cwd(), '.vext-tmp')
  const outputPath = opts.output || tempDirectory

  try {
    fs.ensureDirSync(tempDirectory)
  }
  catch (e) {
    console.log(`临时目录创建失败... ${e.message}`)
    process.exit(1)
  }

  console.log('vendor 列表提取开始')

  const extracter = new VendorExtracter({
    fileModuleFomatter, moduleFilter
  })
  const vendorList = extracter.parse(entries)
  // 获取 vendor 的版本号
  const vendorMap = vendorList.reduce((ret, vendor) => {
    try {
      const main = require.resolve(vendor)
      ret[vendor] = appendVersion ? resolveNpmVersion(main) : ''
    }
    catch (e) {
      console.log(chalk.red(`[Extracter] 找不到模块 ${vendor}`));
    }

    return ret
  }, {})
  const vendorEntryContent = extracter.output(Object.keys(vendorMap))
  const vendorEntryFile = path.join(tempDirectory, 'vendors.js')
  try {
    fs.writeFileSync(vendorEntryFile, vendorEntryContent)
    console.log('vendor 依赖入口已生成完成');
  }
  catch (e) {
    console.log(`写入临时文件失败... ${e.message}`)
    process.exit(1)
  }

  console.log('=======================')
  console.log('webpack 文件生成开始')

  const webpack = require('webpack')
  const FmdTemplatePlugin = require('../webpack-fmd')
  const webpackConfig = {
    entry: {
      vendors: vendorEntryFile
    },
    output: {
      path: outputPath,
      filename: vendorFileNameTemplate,
      publicPath: '/static/'
    },
    module: {
      loaders
    },
    plugins: [
      new FmdTemplatePlugin({
        prefix: vendorPrefix,
        name: 'vendors',
        deps: vendorMap
      })
    ]
  }
  if (prodMode) {
    webpackConfig.devtool = 'source-map'
    webpackConfig.plugins.push(
      new webpack.DefinePlugin({
        'process.env.NODE_ENV': JSON.stringify('production')
      })
    )
  }
  webpack(webpackConfig, (err, stats) => {
    if (err) {
      console.log(`webpack 生成文件失败... ${err.message}`)
    }
    else {
      console.log('任务成功')
    }
  })
}

function resolveNpmVersion (main) {
  try {
    const pkgFile = resolvePkg(main)
    if (pkgFile) {
      const pkg = require(pkgFile)
    }

    return pkg.version
  }
  catch (e) {
    console.log(chalk.red(`[Extracter] 找不到模块 ${moduleName}`));
  }
}

function resolvePkg (main) {
  if (path.basename(main) == 'node_modules' || main == '/') return

  const pkgFile = path.join(main, '../package.json')
  if (fs.existsSync(pkgFile)) return pkgFile
  else {
    return resolvePkg(path.join(main, '..'))
  }
}
